/*
Doug Heffernan v 1.0
(c) 2010 by Viktor "AlgorithMan" Engelmann
www.AlgorithMan.de

This program is licensed under the terms of the GNU General Public License (GPL)
Version 3. See GPL3.txt for further details.

================================================================================

This program finds a distribution of queens on a chess-board such that every
square is threatened. The mathematical/graphtheoretical background of this
program is that it searches a "dominating set" in the "queensgraph".

The dominating-set problem is among the harder problems in NP, it is W[2]-hard,
so it is not fixed parameter tractable in the size of the dominating set (unless
W[2]=FPT, which is very improbable). The problem is FPT in the maximum degree,
but the maximum degree is large in queensgraphs.

So, solving this problem is hard, but this program uses many state-of-the-art
techniques of computer science to solve the problem and it is quite fast.
Hence it is called "Doug Heffernan", which is the name of the protagonist in
the TV show "King of Queens", since this program rules over the problem.

The results are
1 1 1 2 3 3 4 5 5 5 5 6 7 8 9 9 9 9

================================================================================

tried, covered and queen are 2-dimensional vectors (arrays) that store
informations about the currently tried combination of queens.
 - queen[y][x] = true   iff there is a queen on coordinate (x,y)
 - covered[y][x]        sais how many queens threaten the coordinate
 - tried[y][x]          is the height of a level in the recursion tree. If there
                        has been a queen on (x,y) before, then it is useless to
                        put a queen there in a subtree of a sibling of the node
                        in the recursion tree.

bool find(k, uncovered)
-----------------------
This is the main function of the program. It selects a set of
candidate-positions, such that one queen must be put on one of these positions.
It tries each of these positions in different recursive calls and if no solution
is found in one recursive call, then the position is forbidden for the sub-trees
of the other recursive calls (this is where tried[y][x] is used).

there are two possible selections of candidate-sets (the smaller one is used, to
keep the branching-vector small)
1. all positions that can threaten an unthreatened position (here, the position
   is selected that has the smallest amount of candidates to threaten it)
2. if you have X positions left to be threatened, but only k queens left, then
   at least one queen has to threaten X/k positions, so all positions that can
   threaten X/k positions (that are unthreatened so far) form one candidate-set

If there is a "useless queen", then the current branch of the recursion-tree
is aborted, because there would have to be a false selection earlier. A useless
queen is a queen Q for which there is no position that is threatened by Q alone.


findfirst
---------
This is the function that is called by main and that calls find, but the
function itself selects the first position of a queen and exploits the symmetry
of the first choice. For this, it tries the left half of the top row and the
upper half of the upper-left to lower-right diagonal as first choice.


candidates(x,y), newlycovered(x,y)
----------------------------------
count how many positions can threaten (x,y) or how many positions will be
threatened if a queen is put on (x,y). These are used internally to select
the set of candidates


put(x,y,k), unput(x,y)
----------------------
These are used to put a queen to position x,y or remove it from there and
adjust the arrays accordingly.


wipetried(k)
------------
positions are marked after they have been tried in one branch of the recursion-
tree, but for the next sibling, they are allowed again. This ist done by
this function.


print
-----
is only used to display intermediate configurations and the result

*/


#include<vector>
#include<iostream>
#include<stdlib.h>
using namespace std;


vector<vector<int> > tried;
vector<vector<int> > covered;
vector<vector<bool> > queen;
vector<int> xoffset(8); // { 1,-1, 0, 0, 1, 1,-1,-1};
vector<int> yoffset(8); // { 0, 0, 1,-1,-1, 1,-1, 1};
int n;

bool verbose = false;
bool slow = false;
bool printtried = false;

void print() {
	cout << "\33[0;0H"; // home cursor

	if(!printtried) {
		for(int y = 0; y < n; y++) {
			for(int x = 0; x < n; x++)
				cout << (queen[y][x]?" X ":(covered[y][x]==0?" _ ":" # "));
			cout << "\n\n";
		}

	} else { // printtried
		for(int y = 0; y < n; y++) {
			for(int x = 0; x < n; x++)
				cout << (queen[y][x]?" X":(covered[y][x]==0?" _":" #")) << tried[y][x] << " ";
			cout << "\n\n";
		}
	}
	
	if(slow)
		cin.ignore(1);
}


int candidates(int x, int y) {
	int result = (tried[y][x] == 0);
	for(int i = 0; i < 8; i++) {
		int currentx = x+xoffset[i];
		int currenty = y+yoffset[i];
		while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
			result += (tried[currenty][currentx] == 0);
			currentx += xoffset[i];
			currenty += yoffset[i];
		}
	}
	return result;
}


int newlycovered(int x, int y) {
	int result = (covered[y][x]==0);
	for(int i = 0; i < 8; i++) {
		int currentx = x+xoffset[i];
		int currenty = y+yoffset[i];
		while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
			result += (covered[currenty][currentx] == 0);
			currentx += xoffset[i];
			currenty += yoffset[i];
		}
	}
	return result;
}


int put(int x, int y, int k) {
	queen[y][x] = true;
	int result = (covered[y][x]==0);
	covered[y][x]++;
	tried[y][x] = k;
	for(int i = 0; i < 8; i++) {
		int currentx = x+xoffset[i];
		int currenty = y+yoffset[i];
		while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
			result += (covered[currenty][currentx] == 0);
			covered[currenty][currentx]++;
			currentx += xoffset[i];
			currenty += yoffset[i];
		}
	}
	return result;
}


void unput(int x, int y) {
	queen[y][x] = false;
	covered[y][x]--;
	for(int i = 0; i < 8; i++) {
		int currentx = x+xoffset[i];
		int currenty = y+yoffset[i];
		while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
			covered[currenty][currentx]--;
			currentx += xoffset[i];
			currenty += yoffset[i];
		}
	}
}


void wipetried(int k) {
	for(int y = 0; y < n; y++)
		for(int x = 0; x < n; x++)
			if(tried[y][x] == k)
				tried[y][x] = 0;
}


bool uselessqueen() {
	for(int y = 0; y < n; y++) {
		for(int x = 0; x < n; x++) {
			if(!queen[y][x])
				continue;
			
			for(int i = 0; i < 8; i++) {
				int currentx = x;
				int currenty = y;
				while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
					if(covered[currenty][currentx] <= 1)
						return false;
					currentx += xoffset[i];
					currenty += yoffset[i];
				}
			}
		}
	}
	return true;
}


bool find(int k, int uncovered) {
	if(uncovered <= 0)
		return true;
	if(verbose)
		print();
	if(uselessqueen())
		return false;
	

	
	// determine the set of positions that can cover enough positions, that the
	// largest queen can be put there
	int mustcover = uncovered/k;
	int sufficientlylarge = 0;
	vector<int> availsizes(4*n,0);
	for(int y = 0; y < n; y++) {
		for(int x = 0; x < n; x++) {
			int newlycoveredtemp = newlycovered(x,y);
			if(tried[y][x] == 0 && newlycoveredtemp >= mustcover) {
				sufficientlylarge++;
			}
			if(tried[y][x] == 0)
				availsizes[newlycoveredtemp]++;
		}
	}



	// find out, how many uncovered positions CAN be covered by the remaining
	// queens (roughly, a bit overestimated because the lines intersect). If
	// this is less than the number of remaining positions => abort
	int ktemp = k;
	int uncoveredtemp = uncovered;
	for(int i = 4*n-1; uncoveredtemp > 0 && ktemp > 0 && i >= 0; i++) {
		if(ktemp >= availsizes[i]) {
			uncoveredtemp -= availsizes[i]*(i/*-2*/); // --------------------- probably -2 is ok
			ktemp -= availsizes[i];
		} else {
			uncoveredtemp -= ktemp * (i /*-2*/); // --------------------- probably -2 is ok
		}
	}
	if(uncoveredtemp > 0)
		return false;
	
	
	
	// find out, how many positions the smallest queen has to cover (roughly,
	// a bit underestimated)
	ktemp = k-1; // this time 1 step less
	uncoveredtemp = uncovered;
	for(int i = 4*n-1; uncoveredtemp > 0 && ktemp > 0 && i >= 0; i++) {
		if(ktemp >= availsizes[i]) {
			uncoveredtemp -= availsizes[i]*(i/*-2*/); // --------------------- probably -2 is ok
			ktemp -= availsizes[i];
		} else {
			uncoveredtemp -= ktemp * (i /*-2*/); // --------------------- probably -2 is ok
		}
	}
	// exclude all positions that can not cover enough positions
	int mincovered = uncoveredtemp;
	for(int y = 0; y < n; y++) {
		for(int x = 0; x < n; x++) {
			if(tried[y][x] == 0 && newlycovered(x,y) < mincovered) {
				tried[y][x] = k;
			}
		}
	}
	
	
	
	// find out which uncovered position has the least number of positions, that
	// can cover it
	int mincandidates = 4*n-2;
	int selx,sely;
	for(int y = 0; y < n; y++) {
		for(int x = 0; x < n; x++) {
			if(covered[y][x] == 0 && tried[y][x] == 0) {
				int temp = candidates(x,y);
				if(temp < mincandidates) {
					mincandidates = temp;
					selx = x;
					sely = y;
				}
			}
		}
	}



	// now select the smaller set of candidates.
	// 1. The positions large enough to hold the largest queen, or
	// 2. The positions that can threaten (selx,sely) which is the position
	//    with the smallest number of candidates for this
	if(sufficientlylarge < mincandidates) {
		
		for(int y = 0; y < n; y++) {
			for(int x = 0; x < n; x++) {
				if(tried[y][x] == 0 && newlycovered(x,y) >= mustcover) {
					int newlycoveredtemp = put(x, y, k);
					if(find(k-1, uncovered - newlycoveredtemp))
						return true;
					unput(x, y);
				}
			}
		}
		
	} else { // sufficientlylarge <= mincandidates

		for(int i = 0; i < 8; i++) {
			int currentx = selx;
			int currenty = sely;
			while(currentx < n && currenty < n && currentx >= 0 && currenty >= 0) {
				if(tried[currenty][currentx] == 0) {
					int newlycoveredtemp = put(currentx, currenty, k);
					if(find(k-1, uncovered - newlycoveredtemp))
						return true;
					unput(currentx, currenty);
				}
				currentx += xoffset[i];
				currenty += yoffset[i];
			}
		}

	}
	wipetried(k);

	return false;
}


bool findfirst(int k, int uncovered) {
	int n2 = (n-1)/2;
	for(int i = n2; i >= 0; i--) {
		int newlycoveredtemp = put(i,0,k);
		if(find(k-1, uncovered - newlycoveredtemp))
			return true;
		unput(i,0);
		if(!verbose) {
			cout << "|";
			cout.flush();
		}
	}
	for(int i = n2; i > 0; i--) { // notice: here its i>0 instead of i>=0
		int newlycoveredtemp = put(i,i,k);
		if(find(k-1, uncovered - newlycoveredtemp))
			return true;
		unput(i,i);
		if(!verbose) {
			cout << "|";
			cout.flush();
		}
	}
	return false;
}


int main(int argc, char** argv) {
	int k;
	if(argc <= 2) {
		cout << "usage: " << argv[0] << " <board size> <number of queens> [options]\n";
		cout << "   where options may include\n";
		cout << "   -v --verbose  to print intermediate configurations\n";
		cout << "   -t --tried    to print the branch-and-bound matrix (only if also verbose)\n";
		cout << "   -s --slow     to pause after every step\n";
		exit(1);
	} else {
		n = atoi(argv[1]);
		k = atoi(argv[2]);
		for(int i = 3; i < argc; i++) {
			if(string(argv[i]) == "-v" || string(argv[i]) == "--verbose") verbose=true;
			if(string(argv[i]) == "-t" || string(argv[i]) == "--tries") printtried=true;
			if(string(argv[i]) == "-s" || string(argv[i]) == "--slow") slow=true;
		}
	}
	cout << "\33[2J" << "\33["<<1<<";"<<1<<"H"; // clear screen

	tried = vector<vector<int> >(n, vector<int>(n,0));
	covered = vector<vector<int> >(n, vector<int>(n,0));
	queen = vector<vector<bool> >(n, vector<bool>(n,false));
	
	xoffset[0] = 1;  // right
	yoffset[0] = 0;
	xoffset[1] = -1; // left
	yoffset[1] = 0;
	xoffset[2] = 0;  // down
	yoffset[2] = 1;
	xoffset[3] = 0;  // up
	yoffset[3] = -1;
	xoffset[4] = 1;  // up-right
	yoffset[4] = -1;
	xoffset[5] = 1;  // down-right
	yoffset[5] = 1;
	xoffset[6] = -1; // up-left
	yoffset[6] = -1;
	xoffset[7] = -1; // down-left
	yoffset[7] = 1;

	if(!verbose)
		cout << "\33["<<1<<";"<<1<<"H"<<"[" << string(n,' ') << "]" << "\33["<<1<<";"<<1<<"H"<<"[";
	cout.flush();
	
	if(findfirst(k, n*n)) {
		print();
	} else {
		cout << "\33[2J"<<"\33["<<1<<";"<<1<<"H"<<"no solution found\n";
	}

	return 0;
}
